/*
 * Copyright (c) Vicent Marti. All rights reserved.
 *
 * This file is part of clar, distributed under the ISC license.
 * For full terms see the included COPYING file.
 */
#ifndef __CLAR_TEST__
#define __CLAR_TEST__

/* Import the standard clar helper functions */
#include "clar.h"

/* Your custom shared includes / defines here */
extern int global_test_counter;

#endif
